!$
!$=head1 NAME
!$
!$triangle - triangle smoothing
!$
!$=head1 SYNOPSIS
!$
!$initializer- C<call triangle_init(nbox,dat)>
!$
!$operator  - C<int=triangle_lop(adj,add,x,y)>
!$
!$=head1 PARAMETERS
!$
!$=over 4
!$
!$=item nbox - int
!$
!$      size of the box
!$
!$=item ndat - int
!$
!$      number of data points
!$
!$=item adj  - logical
!$
!$      whether or not run adjoint
!$
!$=item add  - logical
!$
!$      wherther or not to add output to input
!$
!$=item x    - C<real(ndat)>
!$
!$      model
!$
!$=item y    - C<real(ndat)>
!$
!$      data
!$
!$=back
!$
!$=head1 DESCRIPTION
!$
!$Triangle smoothing
!$
!$=head1 SEE ALSO
!$
!$L<triangle>,L<triangle2>,L<cdoubint>
!$
!$=head1 LIBRARY
!$
!$B<geef90>
!$
!$=cut
!$
module triangle  
! triangle smoothing
  use adj_mod
  use cdoubint
  use triple
  use fold
  implicit none
  real ,private,  dimension (:), allocatable  :: tmp1 
  real ,private,  dimension (:), allocatable  :: tmp2 
  contains
  subroutine triangle_init ( nbox,ndat )
    integer nbox,ndat,np
    if (.not. allocated(tmp1)) then
      allocate(tmp1 ( ndat+2*nbox)) 
    end if
    if (.not. allocated(tmp2)) then
      allocate(tmp2 ( ndat+2*nbox)) 
    end if
    np = ndat + 2*nbox
    call triple_init (nbox,ndat,-nbox*nbox)
    call cdoubint_init (np)
  end subroutine
  function triangle_lop ( adj, add, x, y) result(stat)
    integer            :: stat 
    logical,intent(in) :: adj,add 
    real,dimension(:)  :: x,y 
    call adjnull (adj,add,x,y )
    call triangle_lop2(adj,add,x,y )
    stat=0
  end function 
  subroutine triangle_lop2(adj,add,x,y)
    logical,intent(in) :: adj,add 
    real, dimension (:)  :: x 
    real, dimension (:)  :: y 
    call chain (fold_lop, cdoubint_lop, triple_lop, adj, .true., x, y,&
      & tmp1, tmp2)
  end subroutine
  subroutine  triangle_close()
    deallocate( tmp1 ,tmp2)
    call cdoubint_close ()
  end subroutine 
end module 
