module wei_data_mod
use wei_wavespace_mod
implicit none
type(sep3d),private,save :: dsep
character(len=128),private,save :: dtag


contains

!-------------------------------------------------!
!                                                 !
!         INITIALIZATION ROUTINES                 !
!                                                 !
!-------------------------------------------------!
logical function init_data(tag,mydsep,created)
logical :: created
type(sep3d) :: mydsep
character(len=*) :: tag

init_data=.false.

if(sep3d_file_exist(tag)) then
  if(.not. init_data_tag(tag,mydsep)) then
    write(0,*) "trouble initializing tag"
    return
  end if
  created=.false.
else
  if(.not. init_data_struct(tag,mydsep)) then
    write(0,*) "trouble initializing structg"
    return
  end if
  created=.true.
end if
init_data=.true.

end function



logical function init_data_struct(tag,mydsep)
character(len=*) :: tag
type(sep3d) :: mydsep,wsep
logical :: myre

init_data_struct=.false.

call  grab_current_wave_space(wsep)
call init_sep3d(mydsep,"SCRATCH","COMPLEX","REGULAR",(/1,1,1,1,1/))
mydsep%n(1:4)=wsep%n(1:4)
mydsep%o(1:4)=wsep%o(1:4)
mydsep%d(1:4)=wsep%d(1:4)
mydsep%label(1)="CMP_X";mydsep%label(2)="CMP_Y"
mydsep%label(3)="H_X";mydsep%label(4)="H_Y"
mydsep%n(5)=wsep%n(6);mydsep%o(5)=wsep%o(6)
mydsep%d(5)=wsep%d(6);mydsep%label(5)="FREQ"
call sep3d_set_sep3d(mydsep)

if(sep_num_thread()>0) then
  mydsep%n(5)=sep_num_thread()
  if(.not. sep3d_section_tag(mydsep,5,"BLOCK",sep_num_thread())) then
    write(0,*) "trouble sectioning tag"
    return
  end if
end if

if(sep_thread_num()==0) call auxinout(tag)
call sep3d_write_description(tag,mydsep)
if(.not. zero_data(tag,mydsep)) then
  write(0,*) "trouble zeroing data"
  return
end if

init_data_struct=.true.
end function

logical function init_data_tag(tag,mydsep)
character(len=*) :: tag
type(sep3d) :: mydsep
init_data_tag=.false.

call init_sep3d(tag,mydsep,"SCRATCH")
init_data_tag=.true.

end function

!-------------------------------------------------!
!                                                 !
!                   IO ROUTINES                   !
!                                                 !
!-------------------------------------------------!

logical function read_data_if(tag,mydsep,i_f,ielem,wfld)
type(sep3d) :: mydsep
character(len=*) :: tag
complex :: wfld(:,:,:,:,:)
integer ::i3,i4,i5,nw(5),fw(5),ndim,nalloc,nloop,i_f,nh,ielem
complex, allocatable :: junkc(:,:)
logical :: ddd,big
read_data_if=.false.
 

nw=mydsep%n;  nw(5)=1; fw=0
if(mydsep%n(1)*mydsep%n(2)< 1000000) then
   big=.true.
   nalloc=mydsep%n(2)*mydsep%n(3)
   nw(3)=mydsep%n(3)
   nloop=1
else
   nalloc=mydsep%n(2)
   nloop=mydsep%n(3)
end if
                                                                                           
allocate(junkc(mydsep%n(1),nalloc))
call add_mem(size(junkc))


fw=0; fw(5)=i_f-1
    do i4=1,mydsep%n(4)
      fw(4)=i4-1
      do i3=1,nloop
        fw(3)=i3-1
        call sep3d_grab_headers(tag,mydsep,nh,nwind=nw(2:),fwind=fw(2:))
        if(.not.sep3d_read_data(tag,mydsep,junkc)) then
          write(0,*) "trouble reading i_f",i_f,sep_thread_num()
          return
        end if
        if(big) then
          wfld(:,:,:,i4,ielem)=reshape(junkc,(/size(wfld,1),size(wfld,2),size(wfld,3)/))
        else
          wfld(:,:,i3,i4,ielem)=junkc
        end if
      end do
    end do
call del_mem(size(junkc))
deallocate(junkc)
read_data_if=.true.
end function


logical function zero_data(tag,mydsep)
type(sep3d) :: mydsep
character(len=*) :: tag
complex, allocatable :: junkc(:,:)
integer ::i3,i4,i5,nw(5),fw(5),ndim,nalloc,nloop,i_f
logical :: ddd,big
zero_data=.false.

                                                                                           
if(mydsep%n(1)*mydsep%n(2)< 1000000) then
   big=.true.
   nalloc=mydsep%n(2)*mydsep%n(3)
   nw(3)=mydsep%n(3)
   nloop=1
else
   nalloc=mydsep%n(2)
   nloop=mydsep%n(3)
end if
                                                                                           
allocate(junkc(mydsep%n(1),nalloc))
call add_mem(size(junkc))
junkc=cmplx(0.,0.)
 

nw=mydsep%n;  nw(5)=1
fw=0;

do i5=1,mydsep%n(5)
  fw(5)=i5-1
  if(sep3d_own(mydsep,i5)) then
    do i4=1,mydsep%n(4)
      fw(4)=i4-1
      do i3=1,nloop
        fw(3)=i3-1
        if(.not.sep3d_write_data(tag,mydsep,junkc,nwind=nw,fwind=fw)) then
          write(0,*) "trouble writing i_f",i_f,sep_thread_num()
          return
        end if
      end do
    end do
  end if
end do
call del_mem(size(junkc))
deallocate(junkc);
zero_data=.true.
end function


logical function write_data_if(tag,mydsep,i_f,ielem,wfld)
type(sep3d) :: mydsep
character(len=*) :: tag
complex :: wfld(:,:,:,:,:)
complex, allocatable :: junkc(:,:)
integer ::i3,i4,i5,nw(5),fw(5),ndim,nalloc,nloop,i_f,nh,ielem
logical :: ddd,big
write_data_if=.false.

                                                                                           
if(mydsep%n(1)*mydsep%n(2)< 1000000) then
   big=.true.
   nalloc=mydsep%n(2)*mydsep%n(3)
   nw(3)=mydsep%n(3)
   nloop=1
else
   nalloc=mydsep%n(2)
   nloop=mydsep%n(3)
end if
                                                                                           
allocate(junkc(mydsep%n(1),nalloc))
call add_mem(size(junkc))

 

nw=mydsep%n;  nw(5)=1
fw=0; fw(5)=i_f-1
    do i4=1,mydsep%n(4)
      fw(4)=i4-1
      do i3=1,nloop
        fw(3)=i3-1
        call sep3d_grab_headers(tag,mydsep,nh,nwind=nw(2:),fwind=fw(2:))
        if(.not.sep3d_read_data(tag,mydsep,junkc)) then
          write(0,*) "trouble reading  i_f",i_f,sep_thread_num()
          return
        end if
        if(big) then
          junkc=junkc+reshape(wfld(:,:,:,i4,ielem),(/size(junkc,1),size(junkc,2)/))
        else
          junkc=junkc+wfld(:,:,i3,i4,ielem)
        end if
        if(.not.sep3d_write_data(tag,mydsep,junkc,nwind=nw,fwind=fw)) then
          write(0,*) "trouble writing i_f",i_f,sep_thread_num()
          return
        end if
      end do
    end do
call del_mem(size(junkc))
deallocate(junkc);
write_data_if=.true.
end function



!-------------------------------------------------!
!                                                 !
!            GRAB/STORE ROUTINES                  !
!                                                 !
!-------------------------------------------------!

subroutine store_current_data_space(d)
type(sep3d) :: d
dsep=d
end subroutine

subroutine grab_current_data_space(d)
type(sep3d) :: d
d=dsep
end subroutine

subroutine grab_current_data_tag(d)
character(len=*) :: d
d=dtag
end subroutine

subroutine store_current_data_tag(d)
character(len=*) :: d
dtag=d
end subroutine












end module
