!$
program zyxabc
implicit none
real, allocatable, dimension (:) :: out
!$=head1 NAME
!$
!$ Scat3d -Create a 3-D scatter model for Kirmod3d
!$
!$=head1 SYNOPSIS
!$
!$ Scat3d [parameters - drho,dlambda,dmu,depth]
!$
!$=head1 INPUT PARAMETERS
!$
!$=over 4
!$
!$=item  mode - int
!$
!$      [1]  Isotropic perturbations (default)
!$      2   lambda, mu & rho perturbations for Rayleigh scattering
!$
!$
!$=item  surface locations given by :
!$
!$    n2,o2,d2-sep_standard model dimension axis 2
!$    n3,o3,d3-sep_standard model dimension axis 3
!$
!$
!$=item dip2    -real
!$
!$      Dip of plane in 2-direction (degrees)
!$
!$=item dip3    -real
!$
!$       3-direction
!$
!$=item depth   -real
!$
!$      [1.]  Depth at physical origin (x=y=0.)
!$
!$=item mag     -real
!$
!$       [1.]  mode=1 Magnitude
!$
!$=item  drho    -real
!$
!$       [0.]
!$
!$=item dlambda -real
!$
!$      [0.]  mode=2 Strength of perturbations
!$
!$=item  dmu     -real
!$
!$      [0.]
!$
!$=back
!$
!$=head1 COMMENTS
!$
!$ Asumes a 3-D model with consistant sampling in all 3 directions.
!$
!$ Output is a n1=4 or 6 seplib array with row 1=n1,
!$                                         row 2=n2,
!$                                         row 3=n3,
!$                              row 4 = mag  or  drho
!$                              row 5 =          dlambda
!$                              row 6 =          dmu
!$
!$=head1  DESCRIPTION
!$
!$ Generates a plane of scatters for use with Kirmod3D modelling program
!$
!$=head1 SEE ALSO
!$
!$L<Marine_geom3d>, L<Scat3d>, L<Gfgradz>
!$
!$=head1 CATEGORY
!$
!$B<seis/model>
!$
!$=cut
!>
integer n1, n2, n3, mode
integer getch,putch
call initpar()
call sep_add_doc_line("NAME")
call sep_add_doc_line("     Scat3d -Create a 3-D scatter model for Kirmod3d")
call sep_add_doc_line("")
call sep_add_doc_line("SYNOPSIS")
call sep_add_doc_line("     Scat3d [parameters - drho,dlambda,dmu,depth]")
call sep_add_doc_line("")
call sep_add_doc_line("INPUT PARAMETERS")
call sep_add_doc_line("    mode - int")
call sep_add_doc_line("              [1]  Isotropic perturbations (default)")
call sep_add_doc_line("              2   lambda, mu & rho perturbations for Rayleigh scattering")
call sep_add_doc_line("")
call sep_add_doc_line("    surface locations given by :")
call sep_add_doc_line("            n2,o2,d2-sep_standard model dimension axis 2")
call sep_add_doc_line("            n3,o3,d3-sep_standard model dimension axis 3")
call sep_add_doc_line("")
call sep_add_doc_line("    dip2 -real")
call sep_add_doc_line("              Dip of plane in 2-direction (degrees)")
call sep_add_doc_line("")
call sep_add_doc_line("    dip3 -real")
call sep_add_doc_line("               3-direction")
call sep_add_doc_line("")
call sep_add_doc_line("    depth -real")
call sep_add_doc_line("              [1.]  Depth at physical origin (x=y=0.)")
call sep_add_doc_line("")
call sep_add_doc_line("    mag -real")
call sep_add_doc_line("               [1.]  mode=1 Magnitude")
call sep_add_doc_line("")
call sep_add_doc_line("    drho -real")
call sep_add_doc_line("               [0.]")
call sep_add_doc_line("")
call sep_add_doc_line("    dlambda -real")
call sep_add_doc_line("              [0.]  mode=2 Strength of perturbations")
call sep_add_doc_line("")
call sep_add_doc_line("    dmu -real")
call sep_add_doc_line("              [0.]")
call sep_add_doc_line("")
call sep_add_doc_line("COMMENTS")
call sep_add_doc_line("     Asumes a 3-D model with consistant sampling in all 3 directions.")
call sep_add_doc_line("")
call sep_add_doc_line("     Output is a n1=4 or 6 seplib array with row 1=n1,")
call sep_add_doc_line("                                             row 2=n2,")
call sep_add_doc_line("                                             row 3=n3,")
call sep_add_doc_line("                                  row 4 = mag  or  drho")
call sep_add_doc_line("                                  row 5 =          dlambda")
call sep_add_doc_line("                                  row 6 =          dmu")
call sep_add_doc_line("")
call sep_add_doc_line("DESCRIPTION")
call sep_add_doc_line("     Generates a plane of scatters for use with Kirmod3D modelling program")
call sep_add_doc_line("")
call sep_add_doc_line("SEE ALSO")
call sep_add_doc_line("    Marine_geom3d, Scat3d, Gfgradz")
call sep_add_doc_line("")
call sep_add_doc_line("CATEGORY")
call sep_add_doc_line("    seis/model")
call sep_add_doc_line("")

call doc('SOURCE')
  if (0>= getch('n2','d',n2 )) then
    call erexit('Trouble reading  n2 from param  ')
  end if
  if (0.ne.putch('Read  from param: #n2 ','d',n2)) then
    call erexit('Trouble writing n2 to history')
  end if
  if (0>= getch('n3','d',n3 )) then
    call erexit('Trouble reading  n3 from param  ')
  end if
  if (0.ne.putch('Read  from param: #n3 ','d',n3)) then
    call erexit('Trouble writing n3 to history')
  end if
  if (0>= getch('mode','d',mode )) then
    mode = 1
  end if
  if (0.ne.putch('Read  from param: #mode ','d',mode)) then
    call erexit('Trouble writing mode to history')
  end if
if (mode.eq.1) then
  n1=4
else
  n1=6
end if 
allocate (out(n1*n2*n3))
call gotoworkbrother (n1,n2,n3,mode,out)
end program zyxabc 
subroutine gotoworkbrother(n1,n2,n3,mode,out)
integer  n1,n2,n3, i1,i2,i3, mode,index
real o1,o2,o3, d1,d2,d3, pi,dip2,dip3, x2,x3,tantheta2,tantheta3
real drho,dlambda,dmu,mag,depth
real out(n1*n2*n3)
integer fetch,getch,putch
  if (0>= fetch('o1','f',o1 )) then
    o1 = 0.
  end if
  if (0>= fetch('o2','f',o2 )) then
    o2 = 0.
  end if
  if (0>= fetch('o3','f',o3 )) then
    o3 = 0.
  end if
  if (0>= getch('d1','f',d1 )) then
    d1 = 1.
  end if
  if (0.ne.putch('Read  from param: #d1 ','f',d1)) then
    call erexit('Trouble writing d1 to history')
  end if
  if (0>= getch('d2','f',d2 )) then
    d2 = 1.
  end if
  if (0.ne.putch('Read  from param: #d2 ','f',d2)) then
    call erexit('Trouble writing d2 to history')
  end if
  if (0>= getch('d3','f',d3 )) then
    d3 = 1.
  end if
  if (0.ne.putch('Read  from param: #d3 ','f',d3)) then
    call erexit('Trouble writing d3 to history')
  end if
  if (0>= getch('dip2','f',dip2 )) then
    dip2 = 0.
  end if
  if (0.ne.putch('Read  from param: #dip2 ','f',dip2)) then
    call erexit('Trouble writing dip2 to history')
  end if
  if (0>= getch('dip3','f',dip3 )) then
    dip3 = 0.
  end if
  if (0.ne.putch('Read  from param: #dip3 ','f',dip3)) then
    call erexit('Trouble writing dip3 to history')
  end if
  if (0>= getch('drho','f',drho )) then
    drho = 0.
  end if
  if (0.ne.putch('Read  from param: #drho ','f',drho)) then
    call erexit('Trouble writing drho to history')
  end if
  if (0>= getch('dlambda','f',dlambda )) then
    dlambda = 0.
  end if
  if (0.ne.putch('Read  from param: #dlambda ','f',dlambda)) then
    call erexit('Trouble writing dlambda to history')
  end if
  if (0>= getch('dmu','f',dmu )) then
    dmu = 0.
  end if
  if (0.ne.putch('Read  from param: #dmu ','f',dmu)) then
    call erexit('Trouble writing dmu to history')
  end if
  if (0>= getch('mag','f',mag )) then
    mag = 1.
  end if
  if (0.ne.putch('Read  from param: #mag ','f',mag)) then
    call erexit('Trouble writing mag to history')
  end if
  if (0>= getch('depth','f',depth )) then
    depth = 1.0
  end if
  if (0.ne.putch('Read  from param: #depth ','f',depth)) then
    call erexit('Trouble writing depth to history')
  end if
pi=4*atan(1.)
tantheta2=tan(dip2*pi/180)
tantheta3=tan(dip3*pi/180)
do i3=1,n3  
  x3 = (i3-1)*d3+o3
  do i2=1,n2  
    x2 = (i2-1)*d2+o2
    index = (i2-1)*n3 + (i3-1)
    out(index*n1+1)=depth+x2*tantheta2+x3*tantheta3
    out(index*n1+2)=x2
    out(index*n1+3)=x3
    if (mode.eq.1) then
      out(index*n1+4)=mag
    else
      out(index*n1+4)=drho
      out(index*n1+5)=dlambda
      out(index*n1+6)=dmu
    end if
  end do
end do 
call srite('out',out,n1*n2*n3*4)
  if (0.ne. putch('n1','d',n1 )) then
    call erexit('Trouble writing  n1 to history  ')
  end if
  if (0.ne. putch('n2','d',n2*n3 )) then
    call erexit('Trouble writing  n2 to history  ')
  end if
  if (0.ne. putch('n3','d',1 )) then
    call erexit('Trouble writing  n3 to history  ')
  end if
  if (0.ne. putch('esize','d',4 )) then
    call erexit('Trouble writing  esize to history  ')
  end if
call hclose()
end   
