/*;
=head1 NAME

 Collect,CollectM
  
=head1 SYNOPSIS

  Collect,CollectM intag= outtag= axis=

=head1 PARAMTERS

=over 4

=item max_size - integer

 [100]  maximum memory in megabyte to use on distribution node (max memory will then be 2x)

=item verb - logical

  [0]  Wether to be verbose

=item master_data - logical

  [1]  Wether the  master node should have any of the input data


=head1 CATEGORY
                                                                                                        
<util/cube>
                                                                                                        

=cut

*/

#include<seplib.h>
#include<sep3dc.h>
#include<sepaux.h>
#include<superset.h>
#ifndef SEPNULL2
#define SEPNULL2 ((void *) 0)
#endif
#ifdef SEP_MPI
#include<mpi.h>
#endif

#define HEAD "/dev/null"
                                                                                                                     
#define MY_SEP_DOC \
 sep_add_doc_line("NAME");\
 sep_add_doc_line("     Collect,CollectM");\
 sep_add_doc_line("");\
 sep_add_doc_line("SYNOPSIS");\
 sep_add_doc_line("      Collect,CollectM intag= outtag= axis=");\
 sep_add_doc_line("");\
 sep_add_doc_line("PARAMTERS");\
 sep_add_doc_line("    max_size - integer");\
 sep_add_doc_line("         [100]  maximum memory in megabyte to use on distribution node (max memory will then be 2x)");\
 sep_add_doc_line("");\
 sep_add_doc_line("    verb - logical");\
 sep_add_doc_line("          [0]  Wether to be verbose");\
 sep_add_doc_line("");\
 sep_add_doc_line("    master_data - logical");\
 sep_add_doc_line("          [1]  Wether the  master node should have any of the input data");\
 sep_add_doc_line("");\
 sep_add_doc_line("CATEGORY");\
 sep_add_doc_line("        <util/cube>");\
 sep_add_doc_line("");

int main(int argc, char **argv){
sep3d input,output;
float *buf;
int   nh,i3,ndo,verb,max_size,n;
int   nsect,nblock,axis,master;
int *nwind,*fwind,*jwind;
char   pattern[256];
char *buf_in,*buf_out;
int esize,ns[2]; 
int master_data,i;
int write_head,write_grid,nhout;
char temp_ch[1024];



#ifdef SEP_MPI
  MPI_Init(&argc,&argv);
   initpar(argc,argv);
  MPI_Comm_size(MPI_COMM_WORLD, &n);
  MPI_Comm_rank(MPI_COMM_WORLD, &i);
  if (i==0) {
    if(0==getch("verb","d",&verb)) verb=0;
    mpi_sep_send_args(n,10,verb);
  }
  else{
     mpi_sep_receive_args();
   }
  if(0==getch("master_data","l",&master_data)) master_data=1;
  if(master_data==0) getch_add_string("intag.master_data=0");
#else
   initpar(argc,argv);
  master_data=1;
  nsect=2;
  if(0==getch("verb","d",&verb)) verb=0;
#endif
  MY_SEP_DOC
  init_3d();
  doc(SOURCE);


/*get parameters*/
if(0==getch("max_size","d",&max_size)) max_size=50;
                                                                                                                     

/*initialize input*/
sep3d_initialize(&input);
if(0!= init_sep3d_tag("intag",&input,"INPUT"))
 seperr("trouble initializing input \n");
if(input.file_format[0]=='G') {write_head=1; write_grid=1;}
else if(input.file_format[0]=='H') {write_head=1; write_grid=0;}
else { write_head=0; write_grid=0;}

if(sep_thread_num()==0){
  if(0!=copy_history("intag","outtag"))
    seperr("trouble copying history file\n");
}

/*initialize output*/
sep3d_initialize(&output);
if(0!= init_sep3d_struct(input,&output,"OUTPUT"))
 seperr("trouble initializing output \n");

if(0!= sep3dc_desection_tag(&output)) seperr("trouble sectioning tag");
                                                                                                                     
if(sep_thread_num()==0) {
  sep3dc_write_description("outtag",&output);
}



if(input.data_format[0]=='C') esize=8;
else if(input.data_format[0]=='B') esize=1;
else esize=4;

max_size=max_size*1000*1000/esize;
init_loop_calc(input.ndims,input.n,"MAIN",max_size);

nwind=(int*) malloc(sizeof(int)*input.ndims);
fwind=(int*) malloc(sizeof(int)*input.ndims);
jwind=(int*) malloc(sizeof(int)*input.ndims);
for(i=0; i < input.ndims; i++) jwind[i]=1;


while(0==do_sep_loop("MAIN",nwind,fwind)){

  if(sep_thread_num()==0 &&verb>0){
    for(i=0; i < input.ndims ; i++)
       fprintf(stderr,"f[%d]=%d n[%d]=%d ",i+1,fwind[i],i+1,nwind[i]);
    fprintf(stderr,"\n");
  }
  if(0!= sep3dc_grab_headers("intag",&input,&nh,&nwind[1],&fwind[1],&jwind[1]))
    seperr("trouble grabbing headers \n");

                                                                                                                     
  if(nh>0) buf_in=(char*) malloc(esize*nh*input.n[0]);
    if(0!=sep3dc_read_data("intag",&input,(void*)buf_in,nwind[0],fwind[0],jwind[0]))
      seperr("trouble reading data");
                                                                                                                     
  if(0!= sep3dc_collect_headers(&input,&output,&nhout)) 
     seperr("trouble collecting headers");


  if(sep_thread_num()==0) buf_out=(char*) malloc(esize*output.n[0]*nhout);

  if(0!=sep3dc_collect_data(&input,&output,buf_in,buf_out)) 
      seperr("trouble collecting headers\n");

  if(sep_thread_num()==0){
    if(0!= sep3dc_write_data("outtag",&output,(char*)buf_out,nwind,fwind,jwind,nh,
     write_head,write_grid))
    seperr("trouble writing outing data");
    free(buf_out); 
  }

  if(nh>0) free(buf_in);

} 
 sep3dc_update_ntraces(&output);
 if(sep_thread_num()==0) sep3d_rite_num_traces("outtag",&output); 

#ifdef SEP_MPI
  MPI_Finalize();
#endif


  return(SUCCESS);
}
