/*$

=head1 NAME

Get - Simple operations on parameters

=head1 SYNOPSIS

Get par=parfile parform=y < Input.h parameter1 parameter2 parameter3 ...
   

=head1 INPUT PARAMETERS

=over 4

=item  parform  -   char 

       [y]-Returns 'parameter1=whatever 
       parameter2=whatever' etc
       according to their values in the input header / parfiles.
       n-Returns the values of the parameters separated by spaces.
 
=back


=head1 DESCRIPTION

Simple operations on parameters


=head1  COMMENTS 

   Example:  Here is a shell to make   n1 = n1 / 2
  		 #/bin/csh -f
  		 set n1 = `Get <junk.H parform=n n1`
  		 @ nh = $n1 / 2
  		 echo n1=$nh

=head1 SEE ALSO

L<Headermath>


=head1 CATEGORY

B<util/info>

=cut
  
 >*/
#include <seplib.h>
#include <string.h>

/*
extern int             sepxargc;
extern char          **sepxargv;
*/

main (argc, argv, envp)
    int             argc;
    char          **argv, **envp;
{
char            string[120];
int             ii;
int             notfirst;
int             parform;

/*
    sepxargc = argc;
    sepxargv = argv;
*/

   initpar(argc,argv);

sep_add_doc_line("NAME");
sep_add_doc_line("    Get - Simple operations on parameters");
sep_add_doc_line("");
sep_add_doc_line("SYNOPSIS");
sep_add_doc_line("    Get par=parfile parform=y < Input.h parameter1 parameter2 parameter3 ...");
sep_add_doc_line("");
sep_add_doc_line("INPUT PARAMETERS");
sep_add_doc_line("    parform - char");
sep_add_doc_line("               [y]-Returns 'parameter1=whatever ");
sep_add_doc_line("               parameter2=whatever' etc");
sep_add_doc_line("               according to their values in the input header / parfiles.");
sep_add_doc_line("               n-Returns the values of the parameters separated by spaces.");
sep_add_doc_line("");
sep_add_doc_line("DESCRIPTION");
sep_add_doc_line("    Simple operations on parameters");
sep_add_doc_line("");
sep_add_doc_line("COMMENTS");
sep_add_doc_line("       Example:  Here is a shell to make   n1 = n1 / 2");
sep_add_doc_line("                     #/bin/csh -f");
sep_add_doc_line("                     set n1 = `Get <junk.H parform=n n1`");
sep_add_doc_line("                     @ nh = $n1 / 2");
sep_add_doc_line("                     echo n1=$nh");
sep_add_doc_line("");
sep_add_doc_line("SEE ALSO");
sep_add_doc_line("    Headermath");
sep_add_doc_line("");
sep_add_doc_line("CATEGORY");
sep_add_doc_line("    util/info");
sep_add_doc_line("");


    doc (SOURCE);

    getch_add_string("head=/dev/null");

    if( !getch("parform","1",&parform) ) parform=1;

    notfirst = 0;

    for (ii = 1; ii < argc; ii++)
    {
	if (notfirst)
	    printf (" ");

	if (NULL == strchr (argv[ii], '='))
	{
	    if( parform ){
	       printf ("%s=", argv[ii]);
            }
	    strcpy (string, "");
	    fetch (argv[ii], "s", string);
	    printf ("%s", string);
	    notfirst = 1;
	}
    }
    printf ("\n");
    return 0;
}
